Package: scTensor
Type: Package
Title: Detection of cell-cell interaction from single-cell RNA-seq
        dataset by tensor decomposition
Version: 2.22.0
Authors@R: c(person("Koki", "Tsuyuzaki", role = c("aut", "cre"),
                email = "k.t.the-answer@hotmail.co.jp"),
            person("Kozo", "Nishida", role = "aut",
                email = "kozo.nishida@gmail.com"))
Depends: R (>= 4.1.0)
Imports: methods, RSQLite, igraph, S4Vectors, plotly, reactome.db,
        AnnotationDbi, SummarizedExperiment, SingleCellExperiment,
        nnTensor (>= 1.1.5), ccTensor (>= 1.0.2), rTensor (>= 1.4.8),
        abind, plotrix, heatmaply, tagcloud, rmarkdown, BiocStyle,
        knitr, AnnotationHub, MeSHDbi (>= 1.29.2), grDevices, graphics,
        stats, utils, outliers, Category, meshr (>= 1.99.1), GOstats,
        ReactomePA, DOSE, crayon, checkmate, BiocManager, visNetwork,
        schex, ggplot2
Suggests: testthat, LRBaseDbi, Seurat, scTGIF, Homo.sapiens
Description: The algorithm is based on the non-negative tucker decomposition (NTD2) of nnTensor.
License: Artistic-2.0
biocViews: DimensionReduction, SingleCell, Software, GeneExpression
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/scTensor
git_branch: RELEASE_3_23
git_last_commit: ec309af
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:36:52 UTC; biocbuild
Author: Koki Tsuyuzaki [aut, cre],
  Kozo Nishida [aut]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
