import(methods)
import(SingleCellExperiment)
import(BiocStyle)
import(knitr)
import(RSQLite)
import(AnnotationHub)

importFrom(BiocManager, install)
importFrom(AnnotationDbi, toTable)
importFrom(igraph, graph.data.frame, bipartite_mapping, max_bipartite_match, graph.empty, add.vertices, add.edges, "E<-", "V<-", V, E, layout_with_dh, plot.igraph, get.edgelist, get.vertex.attribute, edge.attributes)
importFrom(reactome.db, reactomeEXTID2PATHID)
importFrom(MeSHDbi, select)
importFrom(S4Vectors, metadata, "metadata<-")
importFrom(plotly, plot_ly)
importFrom(SummarizedExperiment, assay, assays, "assay<-", "assays<-")
importFrom(rTensor, as.tensor, modeSum, rs_unfold, cs_unfold)
importFrom(abind, abind)
importFrom(plotrix, gradient.rect)
importFrom(heatmaply, heatmaply)
importFrom(tagcloud, smoothPalette, tagcloud)
importFrom(rmarkdown, render)

importFrom(grDevices, colorRampPalette, dev.off, png, rgb)
importFrom(graphics, par, plot, text, legend)
importFrom(utils, browseURL, txtProgressBar, setTxtProgressBar)
importFrom(outliers, grubbs.test, chisq.out.test)
importFrom(nnTensor, NMF, NTD, recTensor)
importFrom(ccTensor, MultiCX)
importFrom(Category, hyperGTest)

importFrom(meshr, meshHyperGTest)
import(GOstats)
importFrom(ReactomePA, enrichPathway)
importFrom(DOSE, enrichDGN, enrichDO, enrichNCG)

importFrom(crayon, bold, blue, green)
importFrom(checkmate, assertClass, assertList)
importFrom(stats, median, cor, dist, na.omit, quantile, p.adjust, hclust, cutree, rbinom, rnbinom, var, sd)
importFrom("utils", "data")
importFrom("grDevices", "hcl")

importFrom(visNetwork, visNetwork)
importFrom(schex, make_hexbin, plot_hexbin_feature)
importFrom(ggplot2, scale_fill_gradient, ggsave)

exportMethods(cellCellSetting, cellCellRanks, cellCellDecomp, cellCellReport, getParam, "setParam<-")
export(cellCellSimulate, newCCSParams)