\name{m}
\alias{m}
\docType{data}
\title{
The gene-wise mean vector of Quartz-Seq data.
}
\description{
This data is internally used in cellCellSimulate function.
}
\usage{data(m)}
\examples{
data(m)
}
\keyword{datasets}