\name{newCCSParams}
\alias{newCCSParams}
\title{New Params}
\usage{
newCCSParams()
}
\arguments{
\item{}{Nothing.}
}
\value{
New Params object.
}
\description{
Create a new CCSParams object.
}
\examples{
params <- newCCSParams()
}