\name{v}
\alias{v}
\docType{data}
\title{
The gene-wise variance vector of Quartz-Seq data.
}
\description{
This data is internally used in cellCellSimulate function.
}
\usage{data(v)}
\examples{
data(v)
}
\keyword{datasets}